@@ -25,8 +25,6 @@
 #endif
 #include "xf86.h"
 #include "xf86_OSproc.h"
-#include "xf86Resources.h"
-#include "xf86RAC.h"
 #include "xf86cmap.h"
 #include "compiler.h"
 #include "mibstore.h"
@@ -46,8 +44,6 @@
 /* framebuffer offscreen manager */
 #include "xf86fbman.h"
 
-/* include xaa includes */
-#include "xaa.h"
 #include "xaarop.h"
 
 /* H/W cursor support */
@@ -57,10 +53,12 @@
 #include "ast.h"
 
 #ifdef	Accel_2D
+typedef Bool (*PFN_bENABLE_CMDQ)(ScrnInfoPtr , ASTRecPtr);
 
 /* Prototype type declaration */
 Bool bInitCMDQInfo(ScrnInfoPtr pScrn, ASTRecPtr pAST);
 Bool bEnableCMDQ(ScrnInfoPtr pScrn, ASTRecPtr pAST);
+Bool bEnableCMDQ2300(ScrnInfoPtr pScrn, ASTRecPtr pAST);
 Bool bEnable2D(ScrnInfoPtr pScrn, ASTRecPtr pAST);
 void vDisable2D(ScrnInfoPtr pScrn, ASTRecPtr pAST);
 void vWaitEngIdle(ScrnInfoPtr pScrn, ASTRecPtr pAST);    
@@ -82,7 +80,7 @@ bInitCMDQInfo(ScrnInfoPtr pScrn, ASTRecPtr pAST)
     if (!pAST->MMIO2D) {
         pAST->CMDQInfo.ulCMDQType = VM_CMD_QUEUE;	
        
-        pScreen = screenInfo.screens[pScrn->scrnIndex];
+        pScreen = xf86ScrnToScreen(pScrn);
       
         do {
             pAST->pCMDQPtr = xf86AllocateOffscreenLinear (pScreen, pAST->CMDQInfo.ulCMDQSize, 8, NULL, NULL, NULL);
@@ -167,7 +165,7 @@ bEnableCMDQ(ScrnInfoPtr pScrn, ASTRecPtr pAST)
         }     
                                  
         *(ULONG *) (pAST->CMDQInfo.pjCmdQBasePort) = ulVMCmdQBasePort;         
-        pAST->CMDQInfo.ulWritePointer = *(ULONG *) (pAST->CMDQInfo.pjWritePort);                 
+        pAST->CMDQInfo.ulWritePointer = *(ULONG *) (pAST->CMDQInfo.pjWritePort) << 3;
         break;
         
     case VM_CMD_MMIO:
@@ -186,10 +184,98 @@ bEnableCMDQ(ScrnInfoPtr pScrn, ASTRecPtr pAST)
     return (TRUE);	
 }
 
+Bool
+bEnableCMDQ2300(ScrnInfoPtr pScrn, ASTRecPtr pAST)
+{
+    ULONG ulVMCmdQBasePort = 0, ulNewModeData;
+
+    vWaitEngIdle(pScrn, pAST);  
+
+    /* set DBG Select Info */
+    if (pAST->DBGSelect)
+    {
+        *(ULONG *) (pAST->MMIOVirtualAddr + 0x804C) = (ULONG) (pAST->DBGSelect);             	
+    }
+    
+    /* set CMDQ base */
+    switch (pAST->CMDQInfo.ulCMDQType)
+    {
+    case VM_CMD_QUEUE:
+        /* enable new CMDQ mode */
+        ulNewModeData = 0xc00000f0;
+        /* set CMDQ Size */
+        switch (pAST->CMDQInfo.ulCMDQSize)
+        {
+        case CMD_QUEUE_SIZE_256K:
+            ulNewModeData |= 0x00000000;   
+            break;
+        	
+        case CMD_QUEUE_SIZE_512K:
+            ulNewModeData |= 0x00000004;   
+            break;
+      
+        case CMD_QUEUE_SIZE_1M:
+            ulNewModeData |= 0x00000008;       
+            break;
+            
+        case CMD_QUEUE_SIZE_2M:
+            ulNewModeData |= 0x0000000C;       
+            break;        
+            
+        default:
+            return(FALSE);
+            break;
+        }     
+        *(ULONG *) (pAST->MMIOVirtualAddr + 0x8060) = ulNewModeData;
+
+        /* Set CMDQ Base */   
+        ulVMCmdQBasePort  = (pAST->CMDQInfo.ulCMDQOffsetAddr - 0) >> 3;                                 
+        *(ULONG *) (pAST->CMDQInfo.pjCmdQBasePort) = ulVMCmdQBasePort;         
+        pAST->CMDQInfo.ulWritePointer = *(ULONG *) (pAST->CMDQInfo.pjWritePort) << 3;                 
+        break;
+        
+    case VM_CMD_MMIO:
+        /* enable new CMDQ mode */
+        ulNewModeData = 0xc00000f2;
+        *(ULONG *) (pAST->MMIOVirtualAddr + 0x8060) = ulNewModeData;
+        break;
+        
+    default:
+        return (FALSE);
+        break;
+    }
+
+    return (TRUE);
+    	
+} /* bEnableCMDQ2300 */
+
 Bool
 bEnable2D(ScrnInfoPtr pScrn, ASTRecPtr pAST)
 {
-    SetIndexRegMask(CRTC_PORT, 0xA4, 0xFE, 0x01);		/* enable 2D */  
+    ULONG ulData;
+    PFN_bENABLE_CMDQ pfnEnableCMDQ = bEnableCMDQ;
+    
+    if (pAST->jChipType == AST2300)
+        pfnEnableCMDQ = bEnableCMDQ2300;
+        	
+    switch (pAST->jChipType)
+    {
+    case AST2100:
+    case AST1100:
+    case AST2200:
+    case AST2150:
+    case AST2300:            
+       *(ULONG *) (pAST->MMIOVirtualAddr + 0xF004) = 0x1e6e0000;
+       *(ULONG *) (pAST->MMIOVirtualAddr + 0xF000) = 0x1;        
+       
+       ulData = *(ULONG *) (pAST->MMIOVirtualAddr + 0x1200c);
+       *(ULONG *) (pAST->MMIOVirtualAddr + 0x1200c) = (ulData & 0xFFFFFFFD);
+
+    case AST2000:                          
+       SetIndexRegMask(CRTC_PORT, 0xA4, 0xFE, 0x01);		/* enable 2D */  
+       
+       break;
+    }
    
     if (!bInitCMDQInfo(pScrn, pAST))
     {
@@ -197,7 +283,7 @@ bEnable2D(ScrnInfoPtr pScrn, ASTRecPtr pAST)
     	return (FALSE);
     }
         
-    if (!bEnableCMDQ(pScrn, pAST))
+    if (!pfnEnableCMDQ(pScrn, pAST))
     {
         vDisable2D(pScrn, pAST);  	
     	return (FALSE);
@@ -213,7 +299,8 @@ vDisable2D(ScrnInfoPtr pScrn, ASTRecPtr pAST)
     vWaitEngIdle(pScrn, pAST);
     vWaitEngIdle(pScrn, pAST);
 
-    SetIndexRegMask(CRTC_PORT, 0xA4, 0xFE, 0x00);		  
+    if (pAST->jChipType != AST1180)
+        SetIndexRegMask(CRTC_PORT, 0xA4, 0xFE, 0x00);		  
 	
 }
 
@@ -229,14 +316,18 @@ vWaitEngIdle(ScrnInfoPtr pScrn, ASTRecPtr pAST)
         ulEngCheckSetting = 0x10000000;
     else
         ulEngCheckSetting = 0x80000000;
+
+    if (pAST->jChipType != AST1180)
+    {
+
+        /* 2D disable if 0xA4 D[0] = 1 */
+        GetIndexRegMask(CRTC_PORT, 0xA4, 0x01, jReg);  
+        if (!jReg) goto Exit_vWaitEngIdle;
     
-    /* 2D disable if 0xA4 D[0] = 1 */
-    GetIndexRegMask(CRTC_PORT, 0xA4, 0x01, jReg);  
-    if (!jReg) goto Exit_vWaitEngIdle;
-    
-    /* 2D not work if in std. mode */
-    GetIndexRegMask(CRTC_PORT, 0xA3, 0x0F, jReg);  
-    if (!jReg) goto Exit_vWaitEngIdle;
+        /* 2D not work if in std. mode */
+        GetIndexRegMask(CRTC_PORT, 0xA3, 0x0F, jReg);  
+        if (!jReg) goto Exit_vWaitEngIdle;
+    }
 
     do  
     {
